@@ -3,12 +3,12 @@
 #include <QHash>
 #include <QList>
 #include <QString>
+#include <memory>
 
 #include "analyzer/analyzer.h"
 #include "analyzer/plugins/analyzerplugin.h"
 #include "preferences/keydetectionsettings.h"
 #include "track/track_decl.h"
-#include "util/memory.h"
 
 class AnalyzerKey : public Analyzer {
   public:
