@@ -1,5 +1,8 @@
 #pragma once
 
+#include <QJSValue>
+#include <utility>
+
 #include "controllers/controller.h"
 #include "controllers/midi/legacymidicontrollermappingfilehandler.h"
 #include "controllers/midi/midimessage.h"
@@ -7,6 +10,19 @@
 
 class MidiOutputHandler;
 
+class MidiInputHandleJSProxy final : public QObject {
+    Q_OBJECT
+  public:
+    MidiInputHandleJSProxy(
+            const std::shared_ptr<LegacyMidiControllerMapping> mapping,
+            const MidiInputMapping& inputMapping);
+    Q_INVOKABLE bool disconnect();
+
+  protected:
+    std::shared_ptr<LegacyMidiControllerMapping> m_mapping;
+    MidiInputMapping m_inputMapping;
+};
+
 /// MIDI Controller base class
 ///
 /// This is a base class representing a MIDI controller.
@@ -51,6 +67,8 @@ class MidiController : public Controller {
         send(data);
     }
 
+    QJSValue makeInputHandler(int status, int midino, const QJSValue& scriptCode);
+
   protected slots:
     virtual void receivedShortMessage(
             unsigned char status,
@@ -60,6 +78,7 @@ class MidiController : public Controller {
     // For receiving System Exclusive messages
     void receive(const QByteArray& data, mixxx::Duration timestamp) override;
     int close() override;
+    void slotBeforeEngineShutdown() override;
 
   private slots:
     bool applyMapping() override;
@@ -118,6 +137,10 @@ class MidiControllerJSProxy : public ControllerJSProxy {
         m_pMidiController->sendSysexMsg(data, length);
     }
 
+    Q_INVOKABLE QJSValue makeInputHandler(int status, int midino, const QJSValue& scriptCode) {
+        return m_pMidiController->makeInputHandler(status, midino, scriptCode);
+    }
+
   private:
     MidiController* m_pMidiController;
 };
