@@ -1,34 +1,36 @@
 #include "controllers/midi/portmidicontroller.h"
 
-#include "controllers/controllerdebug.h"
 #include "controllers/midi/midiutils.h"
 #include "moc_portmidicontroller.cpp"
 
+namespace {
+const QString kUnknownControllerName = QStringLiteral("Unknown PortMidiController");
+} // namespace
+
 PortMidiController::PortMidiController(const PmDeviceInfo* inputDeviceInfo,
         const PmDeviceInfo* outputDeviceInfo,
         int inputDeviceIndex,
-        int outputDeviceIndex,
-        UserSettingsPointer pConfig)
-        : MidiController(pConfig), m_cReceiveMsg_index(0), m_bInSysex(false) {
-    for (unsigned int k = 0; k < MIXXX_PORTMIDI_BUFFER_LEN; ++k) {
-        // Can be shortened to `m_midiBuffer[k] = {}` with C++11.
-        m_midiBuffer[k].message = 0;
-        m_midiBuffer[k].timestamp = 0;
+        int outputDeviceIndex)
+        : MidiController((inputDeviceInfo || outputDeviceInfo)
+                          ? QString::fromLocal8Bit(inputDeviceInfo
+                                            ? inputDeviceInfo->name
+                                            : outputDeviceInfo->name)
+                          : kUnknownControllerName),
+          m_cReceiveMsg_index(0),
+          m_bInSysex(false) {
+    for (int k = 0; k < MIXXX_PORTMIDI_BUFFER_LEN; ++k) {
+        m_midiBuffer[k] = {0, 0};
     }
 
     // Note: We prepend the input stream's index to the device's name to prevent
     // duplicate devices from causing mayhem.
     //setDeviceName(QString("%1. %2").arg(QString::number(m_iInputDeviceIndex), inputDeviceInfo->name));
     if (inputDeviceInfo) {
-        setDeviceName(QString("%1").arg(inputDeviceInfo->name));
         setInputDevice(inputDeviceInfo->input);
         m_pInputDevice.reset(new PortMidiDevice(
             inputDeviceInfo, inputDeviceIndex));
     }
     if (outputDeviceInfo) {
-        if (inputDeviceInfo == nullptr) {
-            setDeviceName(QString("%1").arg(outputDeviceInfo->name));
-        }
         setOutputDevice(outputDeviceInfo->output);
         m_pOutputDevice.reset(new PortMidiDevice(
             outputDeviceInfo, outputDeviceIndex));
@@ -43,7 +45,7 @@ PortMidiController::~PortMidiController() {
 
 int PortMidiController::open() {
     if (isOpen()) {
-        qDebug() << "PortMIDI device" << getName() << "already open";
+        qCWarning(m_logBase) << "PortMIDI device" << getName() << "already open";
         return -1;
     }
 
@@ -55,24 +57,24 @@ int PortMidiController::open() {
     m_cReceiveMsg_index = 0;
 
     if (m_pInputDevice && isInputDevice()) {
-        controllerDebug("PortMidiController: Opening"
-                        << m_pInputDevice->info()->name << "index"
-                        << m_pInputDevice->index() << "for input");
+        qCInfo(m_logBase) << "PortMidiController: Opening"
+                          << m_pInputDevice->info()->name << "index"
+                          << m_pInputDevice->index() << "for input";
         PmError err = m_pInputDevice->openInput(MIXXX_PORTMIDI_BUFFER_LEN);
 
         if (err != pmNoError) {
-            qWarning() << "PortMidi error:" << Pm_GetErrorText(err);
+            qCWarning(m_logBase) << "PortMidi error:" << Pm_GetErrorText(err);
             return -2;
         }
     }
     if (m_pOutputDevice && isOutputDevice()) {
-        controllerDebug("PortMidiController: Opening"
-                        << m_pOutputDevice->info()->name << "index"
-                        << m_pOutputDevice->index() << "for output");
+        qCInfo(m_logBase) << "PortMidiController: Opening"
+                          << m_pOutputDevice->info()->name << "index"
+                          << m_pOutputDevice->index() << "for output";
 
         PmError err = m_pOutputDevice->openOutput();
         if (err != pmNoError) {
-            qWarning() << "PortMidi error:" << Pm_GetErrorText(err);
+            qCWarning(m_logBase) << "PortMidi error:" << Pm_GetErrorText(err);
             return -2;
         }
     }
@@ -84,7 +86,7 @@ int PortMidiController::open() {
 
 int PortMidiController::close() {
     if (!isOpen()) {
-        qDebug() << "PortMIDI device" << getName() << "already closed";
+        qCWarning(m_logBase) << "PortMIDI device" << getName() << "already closed";
         return -1;
     }
 
@@ -96,7 +98,7 @@ int PortMidiController::close() {
     if (m_pInputDevice && m_pInputDevice->isOpen()) {
         PmError err = m_pInputDevice->close();
         if (err != pmNoError) {
-            qWarning() << "PortMidi error:" << Pm_GetErrorText(err);
+            qCWarning(m_logBase) << "PortMidi error:" << Pm_GetErrorText(err);
             result = -1;
         }
     }
@@ -104,7 +106,7 @@ int PortMidiController::close() {
     if (m_pOutputDevice && m_pOutputDevice->isOpen()) {
         PmError err = m_pOutputDevice->close();
         if (err != pmNoError) {
-            qWarning() << "PortMidi error:" << Pm_GetErrorText(err);
+            qCWarning(m_logBase) << "PortMidi error:" << Pm_GetErrorText(err);
             result = -1;
         }
     }
@@ -124,7 +126,7 @@ bool PortMidiController::poll() {
     //qDebug() << "PortMidiController::poll()" << numEvents;
 
     if (numEvents < 0) {
-        qWarning() << "PortMidi error:" << Pm_GetErrorText((PmError)numEvents);
+        qCWarning(m_logInput) << "PortMidi error:" << Pm_GetErrorText((PmError)numEvents);
         return false;
     }
 
@@ -134,7 +136,7 @@ bool PortMidiController::poll() {
 
         if ((status & 0xF8) == 0xF8) {
             // Handle real-time MIDI messages at any time
-            receive(status, 0, 0, timestamp);
+            receivedShortMessage(status, 0, 0, timestamp);
             continue;
         }
 
@@ -148,7 +150,7 @@ bool PortMidiController::poll() {
                 //unsigned char channel = status & 0x0F;
                 unsigned char note = Pm_MessageData1(m_midiBuffer[i].message);
                 unsigned char velocity = Pm_MessageData2(m_midiBuffer[i].message);
-                receive(status, note, velocity, timestamp);
+                receivedShortMessage(status, note, velocity, timestamp);
             }
         }
 
@@ -158,13 +160,15 @@ bool PortMidiController::poll() {
             if (status > 0x7F && status < 0xF7) {
                 m_bInSysex = false;
                 m_cReceiveMsg_index = 0;
-                qWarning() << "Buggy MIDI device: SysEx interrupted!";
+                qCWarning(m_logInput) << "Buggy MIDI device: SysEx interrupted!";
                 goto reprocessMessage;    // Don't lose the new message
             }
 
             // Collect bytes from PmMessage
-            unsigned char data = 0;
-            for (int shift = 0; shift < 32 && (data != MIDI_EOX); shift += 8) {
+            uint8_t data = 0;
+            for (int shift = 0; shift < 32 &&
+                    (data != MidiUtils::opCodeValue(MidiOpCode::EndOfExclusive));
+                    shift += 8) {
                 // TODO(rryan): This prevents buffer overflow if the sysex is
                 // larger than 1024 bytes. I don't want to radically change
                 // anything before the 2.0 release so this will do for now.
@@ -175,7 +179,7 @@ bool PortMidiController::poll() {
             }
 
             // End System Exclusive message if the EOX byte was received
-            if (data == MIDI_EOX) {
+            if (data == MidiUtils::opCodeValue(MidiOpCode::EndOfExclusive)) {
                 m_bInSysex = false;
                 const char* buffer = reinterpret_cast<const char*>(m_cReceiveMsg);
                 receive(QByteArray::fromRawData(buffer, m_cReceiveMsg_index),
@@ -198,30 +202,33 @@ void PortMidiController::sendShortMsg(unsigned char status, unsigned char byte1,
 
     PmError err = m_pOutputDevice->writeShort(word);
     if (err == pmNoError) {
-        controllerDebug(QStringLiteral("outgoing: ") << MidiUtils::formatMidiMessage(getName(),
-                                status,
-                                byte1,
-                                byte2,
-                                MidiUtils::channelFromStatus(status),
-                                MidiUtils::opCodeFromStatus(status)));
+        qCDebug(m_logOutput) << QStringLiteral("outgoing: ")
+                             << MidiUtils::formatMidiOpCode(getName(),
+                                        status,
+                                        byte1,
+                                        byte2,
+                                        MidiUtils::channelFromStatus(status),
+                                        MidiUtils::opCodeFromStatus(status));
     } else {
         // Use two qWarnings() to ensure line break works on all operating systems
-        qWarning() << "Error sending short message"
-                      << MidiUtils::formatMidiMessage(getName(),
-                                                      status, byte1, byte2,
-                                                      MidiUtils::channelFromStatus(status),
-                                                      MidiUtils::opCodeFromStatus(status));
-        qWarning()    << "PortMidi error:" << Pm_GetErrorText(err);
+        qCWarning(m_logOutput) << "Error sending short message"
+                               << MidiUtils::formatMidiOpCode(getName(),
+                                          status,
+                                          byte1,
+                                          byte2,
+                                          MidiUtils::channelFromStatus(status),
+                                          MidiUtils::opCodeFromStatus(status));
+        qCWarning(m_logOutput) << "PortMidi error:" << Pm_GetErrorText(err);
     }
 }
 
-void PortMidiController::send(const QByteArray& data) {
+void PortMidiController::sendBytes(const QByteArray& data) {
     // PortMidi does not receive a length argument for the buffer we provide to
-    // Pm_WriteSysEx. Instead, it scans for a MIDI_EOX byte to know when the
-    // message is over. If one is not provided, it will overflow the buffer and
-    // cause a segfault.
-    if (!data.endsWith(static_cast<char>(MIDI_EOX))) {
-        controllerDebug("SysEx message does not end with 0xF7 -- ignoring.");
+    // Pm_WriteSysEx. Instead, it scans for a MidiOpCode::EndOfExclusive byte
+    // to know when the message is over. If one is not provided, it will
+    // overflow the buffer and cause a segfault.
+    if (!data.endsWith(MidiUtils::opCodeValue(MidiOpCode::EndOfExclusive))) {
+        qCDebug(m_logOutput) << "SysEx message does not end with 0xF7 -- ignoring.";
         return;
     }
 
@@ -231,12 +238,12 @@ void PortMidiController::send(const QByteArray& data) {
 
     PmError err = m_pOutputDevice->writeSysEx((unsigned char*)data.constData());
     if (err == pmNoError) {
-        controllerDebug(QStringLiteral("outgoing: ")
-                << MidiUtils::formatSysexMessage(getName(), data));
+        qCDebug(m_logOutput) << QStringLiteral("outgoing: ")
+                             << MidiUtils::formatSysexMessage(getName(), data);
     } else {
         // Use two qWarnings() to ensure line break works on all operating systems
-        qWarning() << "Error sending SysEx message:"
-                   << MidiUtils::formatSysexMessage(getName(), data);
-        qWarning() << "PortMidi error:" << Pm_GetErrorText(err);
+        qCWarning(m_logOutput) << "Error sending SysEx message:"
+                               << MidiUtils::formatSysexMessage(getName(), data);
+        qCWarning(m_logOutput) << "PortMidi error:" << Pm_GetErrorText(err);
     }
 }
