@@ -55,6 +55,8 @@
     MIXXX_MANUAL_URL "/chapters/user_interface.html#using-cue-modes"
 #define MIXXX_MANUAL_SYNC_MODES_URL \
     MIXXX_MANUAL_URL "/chapters/djing_with_mixxx#sync-lock-with-dynamic-tempo"
+#define MIXXX_MANUAL_TRACK_SEARCH_URL \
+    MIXXX_MANUAL_URL "/chapters/library.html#finding-tracks-search"
 #define MIXXX_MANUAL_BEATS_URL \
     MIXXX_MANUAL_URL "/chapters/preferences.html#beat-detection"
 #define MIXXX_MANUAL_KEY_URL \
