@@ -42,7 +42,7 @@ EngineEffect::EngineEffect(EffectManifestPointer pManifest,
     // At this point the SoundDevice is not set up so we use the kInitalSampleRate.
     const mixxx::EngineParameters engineParameters(
             kInitalSampleRate,
-            MAX_BUFFER_LEN / mixxx::kEngineChannelCount);
+            kMaxEngineFrames);
     m_pProcessor->initialize(activeInputChannels, registeredOutputChannels, engineParameters);
     m_effectRampsFromDry = pManifest->effectRampsFromDry();
 }
@@ -64,7 +64,7 @@ void EngineEffect::initalizeInputChannel(ChannelHandle inputChannel) {
     // At this point the SoundDevice is not set up so we use the kInitalSampleRate.
     const mixxx::EngineParameters engineParameters(
             kInitalSampleRate,
-            MAX_BUFFER_LEN / mixxx::kEngineChannelCount);
+            kMaxEngineFrames);
     m_pProcessor->initializeInputChannel(inputChannel, engineParameters);
 }
 
@@ -130,7 +130,7 @@ bool EngineEffect::process(const ChannelHandle& inputHandle,
         const CSAMPLE* pInput,
         CSAMPLE* pOutput,
         const unsigned int numSamples,
-        const unsigned int sampleRate,
+        const mixxx::audio::SampleRate sampleRate,
         const EffectEnableState chainEnableState,
         const GroupFeatureState& groupFeatures) {
     // Compute the effective enable state from the combination of the effect's state
@@ -181,7 +181,7 @@ bool EngineEffect::process(const ChannelHandle& inputHandle,
     if (effectiveEffectEnableState != EffectEnableState::Disabled) {
         //TODO: refactor rest of audio engine to use mixxx::AudioParameters
         const mixxx::EngineParameters engineParameters(
-                mixxx::audio::SampleRate(sampleRate),
+                sampleRate,
                 numSamples / mixxx::kEngineChannelCount);
 
         m_pProcessor->process(inputHandle,
