@@ -1,5 +1,6 @@
 #pragma once
 
+#include "audio/types.h"
 #include "engine/channelhandle.h"
 #include "engine/effects/message.h"
 #include "util/samplebuffer.h"
@@ -31,7 +32,7 @@ class EngineEffectsManager final : public EffectsRequestHandler {
             const ChannelHandle& outputHandle,
             CSAMPLE* pInOut,
             unsigned int numSamples,
-            unsigned int sampleRate);
+            mixxx::audio::SampleRate sampleRate);
 
     /// Process the postfader EngineEffectChains on the pInOut buffer, modifying
     /// the contents of the input buffer.
@@ -40,7 +41,7 @@ class EngineEffectsManager final : public EffectsRequestHandler {
             const ChannelHandle& outputHandle,
             CSAMPLE* pInOut,
             unsigned int numSamples,
-            unsigned int sampleRate,
+            mixxx::audio::SampleRate sampleRate,
             const GroupFeatureState& groupFeatures,
             CSAMPLE_GAIN oldGain = CSAMPLE_GAIN_ONE,
             CSAMPLE_GAIN newGain = CSAMPLE_GAIN_ONE,
@@ -57,7 +58,7 @@ class EngineEffectsManager final : public EffectsRequestHandler {
             CSAMPLE* pIn,
             CSAMPLE* pOut,
             unsigned int numSamples,
-            unsigned int sampleRate,
+            mixxx::audio::SampleRate sampleRate,
             const GroupFeatureState& groupFeatures,
             CSAMPLE_GAIN oldGain = CSAMPLE_GAIN_ONE,
             CSAMPLE_GAIN newGain = CSAMPLE_GAIN_ONE,
@@ -88,7 +89,7 @@ class EngineEffectsManager final : public EffectsRequestHandler {
             CSAMPLE* pIn,
             CSAMPLE* pOut,
             unsigned int numSamples,
-            unsigned int sampleRate,
+            mixxx::audio::SampleRate sampleRate,
             const GroupFeatureState& groupFeatures,
             CSAMPLE_GAIN oldGain = CSAMPLE_GAIN_ONE,
             CSAMPLE_GAIN newGain = CSAMPLE_GAIN_ONE,
