@@ -13,7 +13,6 @@
 #include "library/trackset/crate/crate.h"
 #include "moc_engineprimeexportjob.cpp"
 #include "track/track.h"
-#include "util/optional.h"
 #include "util/thread_affinity.h"
 #include "waveform/waveformfactory.h"
 
@@ -184,7 +183,7 @@ void exportMetadata(
 
     snapshot.track_number = pTrack->getTrackNumber().toInt();
     if (snapshot.track_number == 0) {
-        snapshot.track_number = djinterop::stdx::nullopt;
+        snapshot.track_number = std::nullopt;
     }
 
     snapshot.duration = std::chrono::milliseconds{
@@ -378,6 +377,10 @@ EnginePrimeExportJob::EnginePrimeExportJob(
     }
 }
 
+// out-of-line declaration because we can't generate dtor in
+// header with unique_ptr's of incomplete types.
+EnginePrimeExportJob::~EnginePrimeExportJob() = default;
+
 void EnginePrimeExportJob::loadIds(const QSet<CrateId>& crateIds) {
     DEBUG_ASSERT_QOBJECT_THREAD_AFFINITY(m_pTrackCollectionManager);
 
@@ -559,7 +562,11 @@ void EnginePrimeExportJob::run() {
             qWarning() << "Failed to export track"
                        << m_pLastLoadedTrack->getId().toString() << ":"
                        << e.what();
-            m_lastErrorMessage = e.what();
+            //: %1 is the artist %2 is the title and %3 is the original error message
+            m_lastErrorMessage = tr("Failed to export track %1 - %2:\n%3")
+                                         .arg(m_pLastLoadedTrack->getArtist(),
+                                                 m_pLastLoadedTrack->getTitle(),
+                                                 e.what());
             emit failed(m_lastErrorMessage);
             return;
         }
