@@ -1,20 +1,18 @@
 #pragma once
 
-#include <QStringListModel>
-#include <QtSql>
 #include <QFuture>
-#include <QtConcurrentRun>
 #include <QFutureWatcher>
 #include <QPointer>
+#include <atomic>
 
 #include "library/baseexternallibraryfeature.h"
-#include "library/trackcollection.h"
-#include "library/treeitemmodel.h"
-#include "library/treeitem.h"
+#include "util/parented_ptr.h"
 
 class BaseExternalTrackModel;
 class BaseExternalPlaylistModel;
 class WLibrarySidebar;
+class ITunesImporter;
+class BaseTrackCache;
 
 class ITunesFeature : public BaseExternalLibraryFeature {
     Q_OBJECT
@@ -24,10 +22,14 @@ class ITunesFeature : public BaseExternalLibraryFeature {
     static bool isSupported();
 
     QVariant title() override;
-    QIcon getIcon() override;
     void bindSidebarWidget(WLibrarySidebar* pSidebarWidget) override;
 
-    TreeItemModel* getChildModel() override;
+    TreeItemModel* sidebarModel() const override;
+
+    // This is called from the ITunesXMLImporter thread and generally threadsafe
+    bool isImportCanceled() const {
+        return m_cancelImport.load();
+    }
 
   public slots:
     void activate() override;
@@ -40,35 +42,37 @@ class ITunesFeature : public BaseExternalLibraryFeature {
     std::unique_ptr<BaseSqlTableModel> createPlaylistModelForPlaylist(
             const QString& playlist) override;
     static QString getiTunesMusicPath();
+    std::unique_ptr<ITunesImporter> makeImporter();
     // returns the invisible rootItem for the sidebar model
     TreeItem* importLibrary();
-    void guessMusicLibraryMountpoint(QXmlStreamReader& xml);
-    void parseTracks(QXmlStreamReader& xml);
-    void parseTrack(QXmlStreamReader& xml, QSqlQuery& query);
-    TreeItem* parsePlaylists(QXmlStreamReader &xml);
-    void parsePlaylist(QXmlStreamReader& xml, QSqlQuery& query1,
-                       QSqlQuery &query2, TreeItem*);
     void clearTable(const QString& table_name);
-    bool readNextStartElement(QXmlStreamReader& xml);
+
+    /// Presents an 'open file' dialog for selecting an iTunes library XML and
+    /// returns the file path.
+    QString showOpenDialog();
+
+    bool isMacOSImporterUsed();
 
     BaseExternalTrackModel* m_pITunesTrackModel;
     BaseExternalPlaylistModel* m_pITunesPlaylistModel;
-    TreeItemModel m_childModel;
+    parented_ptr<TreeItemModel> m_pSidebarModel;
     QStringList m_playlists;
     // a new DB connection for the worker thread
     QSqlDatabase m_database;
-    bool m_cancelImport;
+    std::atomic<bool> m_cancelImport;
     bool m_isActivated;
+
+    /// The path to the iTunes library XML file. If this is empty and the
+    /// user is running macOS, we will use the `ITunesMacOSImporter` that
+    /// uses the native `iTunesLibrary` framework to import the user's
+    /// library. In all other cases, the `ITunesXMLImporter` will be used
+    /// to parse the given file.
     QString m_dbfile;
 
     QFutureWatcher<TreeItem*> m_future_watcher;
     QFuture<TreeItem*> m_future;
     QString m_title;
 
-    QString m_dbItunesRoot;
-    QString m_mixxxItunesRoot;
-
     QSharedPointer<BaseTrackCache> m_trackSource;
     QPointer<WLibrarySidebar> m_pSidebarWidget;
-    QIcon m_icon;
 };
