@@ -124,30 +124,6 @@ void DlgRecording::slotRestoreSearch() {
     emit restoreSearch(currentSearch());
 }
 
-void DlgRecording::activateSelectedTrack() {
-    m_pTrackTableView->activateSelectedTrack();
-}
-
-void DlgRecording::slotAddToAutoDJBottom() {
-    m_pTrackTableView->slotAddToAutoDJBottom();
-}
-
-void DlgRecording::slotAddToAutoDJTop() {
-    m_pTrackTableView->slotAddToAutoDJTop();
-}
-
-void DlgRecording::slotAddToAutoDJReplace() {
-    m_pTrackTableView->slotAddToAutoDJReplace();
-}
-
-void DlgRecording::loadSelectedTrackToGroup(const QString& group, bool play) {
-    m_pTrackTableView->loadSelectedTrackToGroup(group, play);
-}
-
-void DlgRecording::moveSelection(int delta) {
-    m_pTrackTableView->moveSelection(delta);
-}
-
 void DlgRecording::slotRecButtonClicked(bool toggle) {
     Q_UNUSED(toggle);
     m_pRecordingManager->slotToggleRecording(1);
