@@ -2,6 +2,7 @@
 
 #include <QDateTime>
 #include <QMenu>
+#include <QSqlTableModel>
 
 #include "library/library.h"
 #include "library/library_prefs.h"
@@ -13,6 +14,7 @@
 #include "mixer/playerinfo.h"
 #include "moc_setlogfeature.cpp"
 #include "track/track.h"
+#include "util/make_const_iterator.h"
 #include "widget/wlibrary.h"
 #include "widget/wlibrarysidebar.h"
 #include "widget/wtracktableview.h"
@@ -45,8 +47,18 @@ SetlogFeature::SetlogFeature(
     // remove unneeded entries
     deleteAllUnlockedPlaylistsWithFewerTracks();
 
-    // Create empty placeholder playlist for YEAR items
     QString placeholderName = "historyPlaceholder";
+    // remove previously created placeholder playlists
+    const QList<QPair<int, QString>> pls = m_playlistDao.getPlaylists(PlaylistDAO::PLHT_UNKNOWN);
+    QStringList plsToDelete;
+    for (const QPair<int, QString>& pl : pls) {
+        if (pl.second.startsWith(placeholderName)) {
+            plsToDelete.append(QString::number(pl.first));
+        }
+    }
+    m_playlistDao.deletePlaylists(plsToDelete);
+
+    // Create empty placeholder playlist for YEAR items
     m_yearNodeId = m_playlistDao.createUniquePlaylist(&placeholderName,
             PlaylistDAO::PLHT_UNKNOWN);
     DEBUG_ASSERT(m_yearNodeId != kInvalidPlaylistId);
@@ -99,8 +111,10 @@ SetlogFeature::SetlogFeature(
 
 SetlogFeature::~SetlogFeature() {
     // Clean up history when shutting down in case the track threshold changed,
-    // incl. the empty placeholder playlist and potentially empty current playlist
+    // incl. potentially empty current playlist
     deleteAllUnlockedPlaylistsWithFewerTracks();
+    // Delete the placeholder
+    m_playlistDao.deletePlaylist(m_yearNodeId);
 }
 
 QVariant SetlogFeature::title() {
@@ -108,13 +122,13 @@ QVariant SetlogFeature::title() {
 }
 
 void SetlogFeature::bindLibraryWidget(
-        WLibrary* libraryWidget, KeyboardEventFilter* keyboard) {
-    BasePlaylistFeature::bindLibraryWidget(libraryWidget, keyboard);
+        WLibrary* pLibraryWidget, KeyboardEventFilter* pKeyboard) {
+    BasePlaylistFeature::bindLibraryWidget(pLibraryWidget, pKeyboard);
     connect(&PlayerInfo::instance(),
             &PlayerInfo::currentPlayingTrackChanged,
             this,
             &SetlogFeature::slotPlayingTrackChanged);
-    m_libraryWidget = QPointer(libraryWidget);
+    m_pLibraryWidget = QPointer(pLibraryWidget);
 }
 
 void SetlogFeature::deleteAllUnlockedPlaylistsWithFewerTracks() {
@@ -406,10 +420,10 @@ void SetlogFeature::slotJoinWithPrevious() {
 
     // Right-clicked playlist may not be loaded. Use a temporary model to
     // keep sidebar selection and table view in sync
-    QScopedPointer<PlaylistTableModel> pPlaylistTableModel(
-            new PlaylistTableModel(this,
+    std::unique_ptr<PlaylistTableModel> pPlaylistTableModel =
+            std::make_unique<PlaylistTableModel>(this,
                     m_pLibrary->trackCollectionManager(),
-                    "mixxx.db.model.playlist_export"));
+                    "mixxx.db.model.playlist_export");
     pPlaylistTableModel->selectPlaylist(previousPlaylistId);
 
     if (clickedPlaylistId == m_currentPlaylistId) {
@@ -454,10 +468,10 @@ void SetlogFeature::slotMarkAllTracksPlayed() {
 
     // Right-clicked playlist may not be loaded. Use a temporary model to
     // keep sidebar selection and table view in sync
-    QScopedPointer<PlaylistTableModel> pPlaylistTableModel(
-            new PlaylistTableModel(this,
+    std::unique_ptr<PlaylistTableModel> pPlaylistTableModel =
+            std::make_unique<PlaylistTableModel>(this,
                     m_pLibrary->trackCollectionManager(),
-                    "mixxx.db.model.playlist_export"));
+                    "mixxx.db.model.playlist_export");
     pPlaylistTableModel->selectPlaylist(clickedPlaylistId);
     // mark all the Tracks in the previous Playlist as played
     pPlaylistTableModel->select();
@@ -575,14 +589,15 @@ void SetlogFeature::slotPlayingTrackChanged(TrackPointer currentPlayingTrack) {
     if (currentPlayingTrackId.isValid()) {
         // Remove the track from the recent tracks list if it's present and put
         // at the front of the list.
-        auto it = std::find(std::begin(m_recentTracks),
-                std::end(m_recentTracks),
+        const auto it = std::find(
+                m_recentTracks.cbegin(),
+                m_recentTracks.cend(),
                 currentPlayingTrackId);
-        if (it == std::end(m_recentTracks)) {
+        if (it == m_recentTracks.cend()) {
             track_played_recently = false;
         } else {
             track_played_recently = true;
-            m_recentTracks.erase(it);
+            constErase(&m_recentTracks, it);
         }
         m_recentTracks.push_front(currentPlayingTrackId);
 
@@ -615,9 +630,9 @@ void SetlogFeature::slotPlayingTrackChanged(TrackPointer currentPlayingTrack) {
         // View needs a refresh
 
         bool hasActiveView = false;
-        if (m_libraryWidget) {
+        if (m_pLibraryWidget) {
             WTrackTableView* view = dynamic_cast<WTrackTableView*>(
-                    m_libraryWidget->getActiveView());
+                    m_pLibraryWidget->getActiveView());
             if (view != nullptr) {
                 // We have a active view on the history. The user may have some
                 // important active selection. For example putting track into crates
