@@ -1,9 +1,7 @@
 #pragma once
 
 #include <QMimeType>
-#include <QNetworkAccessManager>
 #include <QNetworkReply>
-#include <QPointer>
 #include <QUrl>
 
 #include "network/httpstatuscode.h"
@@ -11,6 +9,8 @@
 #include "util/optional.h"
 #include "util/performancetimer.h"
 
+class QNetworkAccessManager;
+
 namespace mixxx {
 
 namespace network {
