@@ -5,15 +5,8 @@
 #include <QMessageBox>
 #include <QtDebug>
 
-// shout.h checks for WIN32 to see if we are on Windows
-#ifdef WIN64
-#define WIN32
-#endif
 // this is needed to define SHOUT_META_* macros used in version guard
 #include <shoutidjc/shout.h>
-#ifdef WIN64
-#undef WIN32
-#endif
 
 #include "broadcast/defs_broadcast.h"
 #include "control/controlproxy.h"
@@ -153,15 +146,27 @@ DlgPrefBroadcast::DlgPrefBroadcast(QWidget *parent,
              static_cast<int>(EncoderSettings::ChannelMode::STEREO));
 
      connect(checkBoxEnableReconnect,
+#if QT_VERSION >= QT_VERSION_CHECK(6, 7, 0)
+             &QCheckBox::checkStateChanged,
+#else
              &QCheckBox::stateChanged,
+#endif
              this,
              &DlgPrefBroadcast::checkBoxEnableReconnectChanged);
      connect(checkBoxLimitReconnects,
+#if QT_VERSION >= QT_VERSION_CHECK(6, 7, 0)
+             &QCheckBox::checkStateChanged,
+#else
              &QCheckBox::stateChanged,
+#endif
              this,
              &DlgPrefBroadcast::checkBoxLimitReconnectsChanged);
      connect(enableCustomMetadata,
+#if QT_VERSION >= QT_VERSION_CHECK(6, 7, 0)
+             &QCheckBox::checkStateChanged,
+#else
              &QCheckBox::stateChanged,
+#endif
              this,
              &DlgPrefBroadcast::enableCustomMetadataChanged);
 
@@ -287,15 +292,30 @@ void DlgPrefBroadcast::broadcastEnabledChanged(double value) {
     btnDisconnectAll->setEnabled(enabled);
 }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 7, 0)
+void DlgPrefBroadcast::checkBoxEnableReconnectChanged(Qt::CheckState state) {
+    widgetReconnectControls->setEnabled(state == Qt::Checked);
+#else
 void DlgPrefBroadcast::checkBoxEnableReconnectChanged(int value) {
     widgetReconnectControls->setEnabled(value);
+#endif
 }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 7, 0)
+void DlgPrefBroadcast::checkBoxLimitReconnectsChanged(Qt::CheckState state) {
+    spinBoxMaximumRetries->setEnabled(state == Qt::Checked);
+#else
 void DlgPrefBroadcast::checkBoxLimitReconnectsChanged(int value) {
     spinBoxMaximumRetries->setEnabled(value);
+#endif
 }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 7, 0)
+void DlgPrefBroadcast::enableCustomMetadataChanged(Qt::CheckState state) {
+    const bool value = (state == Qt::Checked);
+#else
 void DlgPrefBroadcast::enableCustomMetadataChanged(int value) {
+#endif
     custom_artist->setEnabled(value);
     custom_title->setEnabled(value);
 }
