@@ -1,6 +1,5 @@
 #include "preferences/dialog/dlgpreferences.h"
 
-#include <QDesktopServices>
 #include <QDialog>
 #include <QEvent>
 #include <QMoveEvent>
@@ -16,6 +15,7 @@
 #include "preferences/dialog/dlgpreflibrary.h"
 #include "preferences/dialog/dlgprefsound.h"
 #include "util/color/color.h"
+#include "util/desktophelper.h"
 #include "util/widgethelper.h"
 
 #ifdef __VINYLCONTROL__
@@ -28,9 +28,7 @@
 #include "preferences/dialog/dlgprefeffects.h"
 #include "preferences/dialog/dlgprefinterface.h"
 #include "preferences/dialog/dlgprefmixer.h"
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include "preferences/dialog/dlgprefwaveform.h"
-#endif
 
 #ifdef __BROADCAST__
 #include "preferences/dialog/dlgprefbroadcast.h"
@@ -149,28 +147,25 @@ DlgPreferences::DlgPreferences(
             this,
             &DlgPreferences::reloadUserInterface,
             Qt::DirectConnection);
+    connect(pInterfacePage,
+            &DlgPrefInterface::menuBarAutoHideChanged,
+            this,
+            &DlgPreferences::menuBarAutoHideChanged,
+            Qt::DirectConnection);
     addPageWidget(PreferencesPage(pInterfacePage,
                           new QTreeWidgetItem(
                                   contentsTreeWidget, QTreeWidgetItem::Type)),
             tr("Interface"),
             "ic_preferences_interface.svg");
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     // ugly proxy for determining whether this is being instantiated for QML or legacy QWidgets GUI
     if (pSkinLoader) {
-        DlgPrefWaveform* pWaveformPage = new DlgPrefWaveform(this, m_pConfig, pLibrary);
         addPageWidget(PreferencesPage(
-                              pWaveformPage,
+                              new DlgPrefWaveform(this, m_pConfig, pLibrary),
                               new QTreeWidgetItem(contentsTreeWidget, QTreeWidgetItem::Type)),
                 tr("Waveforms"),
                 "ic_preferences_waveforms.svg");
-        connect(pWaveformPage,
-                &DlgPrefWaveform::reloadUserInterface,
-                this,
-                &DlgPreferences::reloadUserInterface,
-                Qt::DirectConnection);
     }
-#endif
 
     addPageWidget(PreferencesPage(
                           new DlgPrefColors(this, m_pConfig, pLibrary),
@@ -428,6 +423,9 @@ void DlgPreferences::slotButtonPressed(QAbstractButton* pButton) {
     case QDialogButtonBox::AcceptRole:
         // Same as Apply but close the dialog
         emit applyPreferences();
+        // TODO Unfortunately this will accept() even if DlgPrefSound threw a warning
+        // due to inaccessible device(s) or inapplicable samplerate.
+        // https://github.com/mixxxdj/mixxx/issues/6077
         accept();
         break;
     case QDialogButtonBox::RejectRole:
@@ -438,7 +436,7 @@ void DlgPreferences::slotButtonPressed(QAbstractButton* pButton) {
         if (pCurrentPage) {
             QUrl helpUrl = pCurrentPage->helpUrl();
             DEBUG_ASSERT(helpUrl.isValid());
-            QDesktopServices::openUrl(helpUrl);
+            mixxx::DesktopHelper::openUrl(helpUrl);
         }
         break;
     default:
