@@ -5,9 +5,8 @@
 /// Constructs a ChannelGroup.
 /// @param channelBase the first channel in the group.
 /// @param channels the number of channels.
-ChannelGroup::ChannelGroup(unsigned char channelBase, unsigned char channels)
-  : m_channelBase(channelBase)
-  , m_channels(channels) {
+ChannelGroup::ChannelGroup(unsigned char channelBase, mixxx::audio::ChannelCount channels)
+        : m_channelBase(channelBase), m_channels(channels) {
 }
 
 /// @return This ChannelGroup's base channel
@@ -16,7 +15,7 @@ unsigned char ChannelGroup::getChannelBase() const {
 }
 
 /// @return The number of channels in this ChannelGroup
-unsigned char ChannelGroup::getChannelCount() const {
+mixxx::audio::ChannelCount ChannelGroup::getChannelCount() const {
     return m_channels;
 }
 
@@ -25,7 +24,7 @@ unsigned char ChannelGroup::getChannelCount() const {
 /// @return true if the other and this ChannelGroup share any channels,
 ///          false otherwise.
 bool ChannelGroup::clashesWith(const ChannelGroup &other) const {
-    if (m_channels == 0 || other.m_channels == 0) {
+    if (!m_channels.isValid() || !other.m_channels.isValid()) {
         return false; // can't clash if there are no channels in use
     }
     return (m_channelBase > other.m_channelBase
@@ -40,7 +39,7 @@ bool ChannelGroup::clashesWith(const ChannelGroup &other) const {
 /// constructor, AudioPath is abstract).
 /// @param channelBase the first channel on a sound device used by this AudioPath.
 /// @param channels the number of channels used.
-AudioPath::AudioPath(unsigned char channelBase, unsigned char channels)
+AudioPath::AudioPath(unsigned char channelBase, mixxx::audio::ChannelCount channels)
         : m_channelGroup(channelBase, channels),
           m_type(AudioPathType::Invalid),
           m_index(0) {
@@ -202,27 +201,27 @@ AudioPathType AudioPath::getTypeFromInt(int typeInt) {
 }
 
 // static
-unsigned char AudioPath::minChannelsForType(AudioPathType type) {
+mixxx::audio::ChannelCount AudioPath::minChannelsForType(AudioPathType type) {
     switch (type) {
     case AudioPathType::VinylControl:
-        return 2;
+        return mixxx::audio::ChannelCount::stereo();
     default:
-        return 1;
+        return mixxx::audio::ChannelCount::mono();
     }
 }
 
 // static
-unsigned char AudioPath::maxChannelsForType(AudioPathType type) {
+mixxx::audio::ChannelCount AudioPath::maxChannelsForType(AudioPathType type) {
     Q_UNUSED(type);
-    return 2;
+    return mixxx::audio::ChannelCount::stereo();
 }
 
 /// Constructs an AudioOutput.
 AudioOutput::AudioOutput(AudioPathType type,
-                         unsigned char channelBase,
-                         unsigned char channels,
-                         unsigned char index)
-    : AudioPath(channelBase, channels) {
+        unsigned char channelBase,
+        mixxx::audio::ChannelCount channels,
+        unsigned char index)
+        : AudioPath(channelBase, channels) {
     // TODO(rryan): This is a virtual function call from a constructor.
     setType(type);
     if (isIndexed(type)) {
@@ -239,7 +238,7 @@ QDomElement AudioOutput::toXML(QDomElement *element) const {
     element->setAttribute("type", AudioPath::getStringFromType(m_type));
     element->setAttribute("index", m_index);
     element->setAttribute("channel", m_channelGroup.getChannelBase());
-    element->setAttribute("channel_count", m_channelGroup.getChannelCount());
+    element->setAttribute("channel_count", m_channelGroup.getChannelCount().value());
     return *element;
 }
 
@@ -249,13 +248,14 @@ AudioOutput AudioOutput::fromXML(const QDomElement &xml) {
     AudioPathType type(AudioPath::getTypeFromString(xml.attribute("type")));
     unsigned int index(xml.attribute("index", "0").toUInt());
     unsigned int channel(xml.attribute("channel", "0").toUInt());
-    unsigned int channels(xml.attribute("channel_count", "0").toUInt());
+    auto channels = mixxx::audio::ChannelCount::fromInt(
+            xml.attribute("channel_count", "0").toInt());
     // In Mixxx < 1.12.0 we didn't save channels to file since they directly
     // corresponded to the type. To migrate users over, use mono for all
     // microphones and stereo for everything else since previously microphone
     // inputs were the only mono AudioPath.
-    if (channels == 0) {
-        channels = 2;
+    if (!channels.isValid()) {
+        channels = mixxx::audio::ChannelCount::stereo();
     }
     return AudioOutput(type, channel, channels, index);
 }
@@ -286,9 +286,9 @@ void AudioOutput::setType(AudioPathType type) {
 
 /// Constructs an AudioInput.
 AudioInput::AudioInput(AudioPathType type,
-                       unsigned char channelBase,
-                       unsigned char channels,
-                       unsigned char index)
+        unsigned char channelBase,
+        mixxx::audio::ChannelCount channels,
+        unsigned char index)
         : AudioPath(channelBase, channels) {
     // TODO(rryan): This is a virtual function call from a constructor.
     setType(type);
@@ -310,7 +310,7 @@ QDomElement AudioInput::toXML(QDomElement *element) const {
     element->setAttribute("type", AudioPath::getStringFromType(m_type));
     element->setAttribute("index", m_index);
     element->setAttribute("channel", m_channelGroup.getChannelBase());
-    element->setAttribute("channel_count", m_channelGroup.getChannelCount());
+    element->setAttribute("channel_count", m_channelGroup.getChannelCount().value());
     return *element;
 }
 
@@ -320,13 +320,15 @@ AudioInput AudioInput::fromXML(const QDomElement &xml) {
     AudioPathType type(AudioPath::getTypeFromString(xml.attribute("type")));
     unsigned int index(xml.attribute("index", "0").toUInt());
     unsigned int channel(xml.attribute("channel", "0").toUInt());
-    unsigned int channels(xml.attribute("channel_count", "0").toUInt());
+    auto channels = mixxx::audio::ChannelCount::fromInt(
+            xml.attribute("channel_count", "0").toInt());
     // In Mixxx <1.12.0 we didn't save channels to file since they directly
     // corresponded to the type. To migrate users over, use mono for all
     // microphones and stereo for everything else since previously microphone
     // inputs were the only mono AudioPath.
-    if (channels == 0) {
-        channels = type == AudioPathType::Microphone ? 1 : 2;
+    if (!channels.isValid()) {
+        channels = type == AudioPathType::Microphone ? mixxx::audio::ChannelCount::mono()
+                                                     : mixxx::audio::ChannelCount::stereo();
     }
     return AudioInput(type, channel, channels, index);
 }
