@@ -284,7 +284,10 @@ bool SoundSourceProxy::isUrlSupported(const QUrl& url) {
 
 // static
 bool SoundSourceProxy::isFileSupported(const mixxx::FileInfo& fileInfo) {
-    return isFileNameSupported(fileInfo.fileName());
+    const QString fileType =
+            mixxx::SoundSource::getTypeFromFile(fileInfo.asQFileInfo());
+    qDebug() << "isFileSupported" << fileType;
+    return isFileTypeSupported(fileType);
 }
 
 // static
@@ -500,7 +503,7 @@ void SoundSourceProxy::findProviderAndInitSoundSource() {
     if (!getUrl().isEmpty()) {
         kLogger.warning()
                 << "No SoundSourceProvider for file"
-                << getUrl().toString();
+                << getUrl().toString(QUrl::PreferLocalFile);
     }
 }
 
@@ -514,7 +517,7 @@ bool SoundSourceProxy::initSoundSourceWithProvider(
         kLogger.warning() << "SoundSourceProvider"
                           << pProvider->getDisplayName()
                           << "failed to create a SoundSource for file"
-                          << getUrl().toString();
+                          << getUrl().toString(QUrl::PreferLocalFile);
         return false;
     }
     m_pProvider = pProvider;
@@ -522,7 +525,7 @@ bool SoundSourceProxy::initSoundSourceWithProvider(
         kLogger.debug() << "SoundSourceProvider"
                         << m_pProvider->getDisplayName()
                         << "created a SoundSource for file"
-                        << getUrl().toString()
+                        << getUrl().toString(QUrl::PreferLocalFile)
                         << "of type"
                         << m_pSoundSource->getType();
     }
@@ -625,7 +628,7 @@ SoundSourceProxy::UpdateTrackFromSourceResult SoundSourceProxy::updateTrackFromS
     if (!m_pSoundSource) {
         kLogger.warning()
                 << "Unable to update track from unsupported file type"
-                << getUrl().toString();
+                << getUrl().toString(QUrl::PreferLocalFile);
         return UpdateTrackFromSourceResult::NotUpdated;
     }
 
@@ -641,7 +644,7 @@ SoundSourceProxy::UpdateTrackFromSourceResult SoundSourceProxy::updateTrackFromS
                           << "to"
                           << newType
                           << "for file"
-                          << getUrl().toString();
+                          << getUrl().toString(QUrl::PreferLocalFile);
     }
 
     // Use the existing track metadata as default values. Otherwise
@@ -653,6 +656,13 @@ SoundSourceProxy::UpdateTrackFromSourceResult SoundSourceProxy::updateTrackFromS
     mixxx::TrackMetadata trackMetadata =
             m_pTrack->getMetadata(&sourceSyncStatus);
 
+    if (sourceSyncStatus == mixxx::TrackRecord::SourceSyncStatus::Undefined) {
+        kLogger.warning()
+                << "Unable to update track from missing or inaccessible file"
+                << getUrl().toString();
+        return UpdateTrackFromSourceResult::NotUpdated;
+    }
+
     // Save for later to replace the unreliable and imprecise audio
     // properties imported from file tags (see below).
     const auto preciseStreamInfo = trackMetadata.getStreamInfo();
@@ -677,7 +687,7 @@ SoundSourceProxy::UpdateTrackFromSourceResult SoundSourceProxy::updateTrackFromS
             if (kLogger.debugEnabled()) {
                 kLogger.debug()
                         << "Skip importing of embedded cover art from file"
-                        << getUrl().toString();
+                        << getUrl().toString(QUrl::PreferLocalFile);
             }
         } else {
             // Request reimport of embedded cover art
@@ -703,7 +713,7 @@ SoundSourceProxy::UpdateTrackFromSourceResult SoundSourceProxy::updateTrackFromS
                 << "Failed to import track metadata"
                 << (pCoverImg ? "and embedded cover art" : "")
                 << "from file"
-                << getUrl().toString();
+                << getUrl().toString(QUrl::PreferLocalFile);
         // make sure that the trackMetadata was not messed up due to the failure
         mixxx::TrackRecord::SourceSyncStatus sourceSyncStatusNew;
         trackMetadata = m_pTrack->getMetadata(&sourceSyncStatusNew);
@@ -747,7 +757,7 @@ SoundSourceProxy::UpdateTrackFromSourceResult SoundSourceProxy::updateTrackFromS
         if (kLogger.debugEnabled()) {
             kLogger.debug()
                     << "Initializing track metadata and embedded cover art from file"
-                    << getUrl().toString();
+                    << getUrl().toString(QUrl::PreferLocalFile);
         }
     } else {
         if (kLogger.debugEnabled()) {
@@ -755,7 +765,7 @@ SoundSourceProxy::UpdateTrackFromSourceResult SoundSourceProxy::updateTrackFromS
                     << "Re-importing track metadata"
                     << (pCoverImg ? "and embedded cover art" : "")
                     << "from file"
-                    << getUrl().toString();
+                    << getUrl().toString(QUrl::PreferLocalFile);
         }
     }
 
@@ -867,14 +877,14 @@ bool SoundSourceProxy::openSoundSource(
             }
             kLogger.warning()
                     << "Failed to read file"
-                    << getUrl().toString()
+                    << getUrl().toString(QUrl::PreferLocalFile)
                     << "with provider"
                     << m_pProvider->getDisplayName();
             m_pSoundSource->close(); // cleanup
         } else {
             kLogger.warning()
                     << "Failed to open file"
-                    << getUrl().toString()
+                    << getUrl().toString(QUrl::PreferLocalFile)
                     << "with provider"
                     << m_pProvider->getDisplayName()
                     << "using mode"
@@ -912,7 +922,7 @@ bool SoundSourceProxy::openSoundSource(
     // getting here. m_pSoundSource might already be invalid/null!
     kLogger.warning()
             << "Giving up to open file"
-            << getUrl().toString()
+            << getUrl().toString(QUrl::PreferLocalFile)
             << "after"
             << attemptCount
             << "unsuccessful attempts";
