@@ -1,9 +1,10 @@
 #include <benchmark/benchmark.h>
 #include <gtest/gtest.h>
 
-#include <QtDebug>
 #include <QList>
 #include <QPair>
+#include <QtDebug>
+#include <vector>
 
 #include "util/sample.h"
 #include "util/timer.h"
@@ -260,12 +261,12 @@ TEST_F(SampleUtilTest, convertS16ToFloat32) {
     for (int i = 0; i < buffers.size(); ++i) {
         CSAMPLE* buffer = buffers[i];
         int size = sizes[i];
-        SAMPLE* s16 = new SAMPLE[size];
+        auto s16 = std::vector<SAMPLE>(size);
         FillBuffer(buffer, 1.0f, size);
         for (int j = 0; j < size; ++j) {
             s16[j] = SAMPLE_MAXIMUM;
         }
-        SampleUtil::convertS16ToFloat32(buffer, s16, size);
+        SampleUtil::convertS16ToFloat32(buffer, s16.data(), size);
         for (int j = 0; j < size; ++j) {
             EXPECT_FLOAT_EQ(expectedMax, buffer[j]);
         }
@@ -273,7 +274,7 @@ TEST_F(SampleUtilTest, convertS16ToFloat32) {
         for (int j = 0; j < size; ++j) {
             s16[j] = 0;
         }
-        SampleUtil::convertS16ToFloat32(buffer, s16, size);
+        SampleUtil::convertS16ToFloat32(buffer, s16.data(), size);
         for (int j = 0; j < size; ++j) {
             EXPECT_FLOAT_EQ(0.0f, buffer[j]);
         }
@@ -281,11 +282,10 @@ TEST_F(SampleUtilTest, convertS16ToFloat32) {
         for (int j = 0; j < size; ++j) {
             s16[j] = SAMPLE_MINIMUM;
         }
-        SampleUtil::convertS16ToFloat32(buffer, s16, size);
+        SampleUtil::convertS16ToFloat32(buffer, s16.data(), size);
         for (int j = 0; j < size; ++j) {
             EXPECT_FLOAT_EQ(-1.0f, buffer[j]);
         }
-        delete [] s16;
     }
 }
 
