@@ -5,6 +5,7 @@
 
 #include <QTest>
 #include <QtDebug>
+#include <memory>
 
 #include "control/controlindicatortimer.h"
 #include "control/controlobject.h"
@@ -25,7 +26,6 @@
 #include "test/soundsourceproviderregistration.h"
 #include "track/track.h"
 #include "util/defs.h"
-#include "util/memory.h"
 #include "util/sample.h"
 #include "util/types.h"
 
