@@ -11,6 +11,7 @@
 #include <QCoreApplication>
 #include <QProcessEnvironment>
 #include <QStandardPaths>
+#include <QtGlobal>
 
 #include "config.h"
 #include "defs_urls.h"
@@ -23,27 +24,37 @@ bool calcUseColorsAuto() {
     // see https://no-color.org/
     if (QProcessEnvironment::systemEnvironment().contains(QLatin1String("NO_COLOR"))) {
         return false;
-    } else {
-#ifndef __WINDOWS__
-        if (isatty(fileno(stderr))) {
-            return true;
-        }
-#else
-        if (_isatty(_fileno(stderr))) {
-            return true;
-        }
-#endif
     }
-    return false;
+
+#ifndef __WINDOWS__
+    if (!isatty(fileno(stderr))) {
+        return false;
+    }
+#else
+    if (!_isatty(_fileno(stderr))) {
+        return false;
+    }
+#endif
+
+    // Check if terminal is known to support ANSI colors
+    QString term = QProcessEnvironment::systemEnvironment().value("TERM");
+    return term == "alacritty" || term == "ansi" || term == "cygwin" || term == "linux" ||
+            term.startsWith("screen") || term.startsWith("xterm") ||
+            term.startsWith("vt100") || term.startsWith("rxvt") ||
+            term.endsWith("color");
 }
 
 } // namespace
 
 CmdlineArgs::CmdlineArgs()
         : m_startInFullscreen(false), // Initialize vars
+          m_startAutoDJ(false),
           m_controllerDebug(false),
           m_controllerAbortOnWarning(false),
           m_developer(false),
+#ifdef MIXXX_USE_QML
+          m_qml(false),
+#endif
           m_safeMode(false),
           m_useLegacyVuMeter(false),
           m_useLegacySpinny(false),
@@ -54,20 +65,37 @@ CmdlineArgs::CmdlineArgs()
           m_parseForUserFeedbackRequired(false),
           m_logLevel(mixxx::kLogLevelDefault),
           m_logFlushLevel(mixxx::kLogFlushLevelDefault),
+          m_logMaxFileSize(mixxx::kLogMaxFileSizeDefault),
 // We are not ready to switch to XDG folders under Linux, so keeping $HOME/.mixxx as preferences folder. see #8090
 #ifdef MIXXX_SETTINGS_PATH
-          m_settingsPath(QDir::homePath().append("/").append(MIXXX_SETTINGS_PATH)) {
-#else
-#ifdef __LINUX__
+          m_settingsPath(QDir::homePath().append("/").append(MIXXX_SETTINGS_PATH))
+#elif defined(__LINUX__)
 #error "We are not ready to switch to XDG folders under Linux"
-#endif
+#elif defined(Q_OS_IOS)
+          // On iOS we intentionally use a user-accessible subdirectory of the sandbox
+          // documents directory rather than the default app data directory. Specifically
+          // we use
+          //
+          //     <sandbox home>/Documents/Library/Application Support/Mixxx
+          //
+          // instead of the default (and hidden)
+          //
+          //     <sandbox home>/Library/Application Support/Mixxx
+          //
+          // This lets the user back up their mixxxdb, add custom controller mappings,
+          // potentially diagnose issues by accessing logs etc. via the native iOS files app.
+          m_settingsPath(
+                  QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation)
+                          .append("/Library/Application Support/Mixxx"))
+#else
 
           // TODO(XXX) Trailing slash not needed anymore as we switches from String::append
           // to QDir::filePath elsewhere in the code. This is candidate for removal.
           m_settingsPath(
                   QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation)
-                          .append("/")) {
+                          .append("/"))
 #endif
+{
 }
 
 namespace {
@@ -154,6 +182,12 @@ bool CmdlineArgs::parse(const QStringList& arguments, CmdlineArgs::ParseMode mod
             QStringLiteral("locale"));
     parser.addOption(locale);
 
+    const QCommandLineOption startAutoDJ(QStringLiteral("start-autodj"),
+            forUserFeedback ? QCoreApplication::translate("CmdlineArgs",
+                                      "Starts Auto DJ when Mixxx is launched.")
+                            : QString());
+    parser.addOption(startAutoDJ);
+
     // An option with a value
     const QCommandLineOption settingsPath(QStringLiteral("settings-path"),
             forUserFeedback ? QCoreApplication::translate("CmdlineArgs",
@@ -237,6 +271,13 @@ bool CmdlineArgs::parse(const QStringList& arguments, CmdlineArgs::ParseMode mod
                             : QString());
     parser.addOption(developer);
 
+#ifdef MIXXX_USE_QML
+    const QCommandLineOption qml(QStringLiteral("qml"),
+            forUserFeedback ? QCoreApplication::translate("CmdlineArgs",
+                                      "Loads experimental QML GUI instead of legacy QWidget skin")
+                            : QString());
+    parser.addOption(qml);
+#endif
     const QCommandLineOption safeMode(QStringLiteral("safe-mode"),
             forUserFeedback ? QCoreApplication::translate("CmdlineArgs",
                                       "Enables safe-mode. Disables OpenGL waveforms, and "
@@ -286,6 +327,18 @@ bool CmdlineArgs::parse(const QStringList& arguments, CmdlineArgs::ParseMode mod
     parser.addOption(logFlushLevel);
     parser.addOption(logFlushLevelDeprecated);
 
+    const QCommandLineOption logMaxFileSize(QStringLiteral("log-max-file-size"),
+            forUserFeedback ? QCoreApplication::translate("CmdlineArgs",
+                                      "Sets the maximum file size of the "
+                                      "mixxx.log file in bytes. "
+                                      "Use -1 for unlimited. The default is "
+                                      "100 MB as 1e5 or 100000000.")
+                            : QString(),
+            QStringLiteral("bytes"));
+    logFlushLevelDeprecated.setFlags(QCommandLineOption::HiddenFromHelp);
+    logFlushLevelDeprecated.setValueName(logFlushLevel.valueName());
+    parser.addOption(logMaxFileSize);
+
     QCommandLineOption debugAssertBreak(QStringLiteral("debug-assert-break"),
             forUserFeedback ? QCoreApplication::translate("CmdlineArgs",
                                       "Breaks (SIGINT) Mixxx, if a DEBUG_ASSERT evaluates to "
@@ -341,6 +394,10 @@ bool CmdlineArgs::parse(const QStringList& arguments, CmdlineArgs::ParseMode mod
         m_locale = parser.value(locale);
     }
 
+    if (parser.isSet(startAutoDJ)) {
+        m_startAutoDJ = true;
+    }
+
     if (parser.isSet(settingsPath)) {
         m_settingsPath = parser.value(settingsPath);
         if (!m_settingsPath.endsWith("/")) {
@@ -372,6 +429,9 @@ bool CmdlineArgs::parse(const QStringList& arguments, CmdlineArgs::ParseMode mod
     m_controllerDebug = parser.isSet(controllerDebug) || parser.isSet(controllerDebugDeprecated);
     m_controllerAbortOnWarning = parser.isSet(controllerAbortOnWarning);
     m_developer = parser.isSet(developer);
+#ifdef MIXXX_USE_QML
+    m_qml = parser.isSet(qml);
+#endif
     m_safeMode = parser.isSet(safeMode) || parser.isSet(safeModeDeprecated);
     m_debugAssertBreak = parser.isSet(debugAssertBreak) || parser.isSet(debugAssertBreakDeprecated);
 
@@ -409,6 +469,17 @@ bool CmdlineArgs::parse(const QStringList& arguments, CmdlineArgs::ParseMode mod
         }
     }
 
+    if (parser.isSet(logMaxFileSize)) {
+        QString strLogMaxFileSize = parser.value(logMaxFileSize);
+        bool ok = false;
+        // We parse it as double to also support exponential notation
+        m_logMaxFileSize = static_cast<qint64>(strLogMaxFileSize.toDouble(&ok));
+        if (!ok) {
+            fputs("\nFailed to parse log-max-file-size.\n", stdout);
+            return false;
+        }
+    }
+
     // set colors
     if (parser.value(color).compare(QLatin1String("always"), Qt::CaseInsensitive) == 0) {
         m_useColors = true;
