@@ -8,16 +8,17 @@
 #endif
 
 #include <cmath>
+#include <limits>
 
 int util_fpclassify(float x) {
     return std::fpclassify(x);
 }
 
-int util_isfinite(float x) {
+bool util_isfinite(float x) {
     return std::isfinite(x);
 }
 
-int util_isnormal(float x) {
+bool util_isnormal(float x) {
     return std::isnormal(x);
 }
 
@@ -33,11 +34,11 @@ int util_fpclassify(double x) {
     return std::fpclassify(x);
 }
 
-int util_isfinite(double x) {
+bool util_isfinite(double x) {
     return std::isfinite(x);
 }
 
-int util_isnormal(double x) {
+bool util_isnormal(double x) {
     return std::isnormal(x);
 }
 
@@ -48,3 +49,11 @@ int util_isnan(double x) {
 int util_isinf(double x) {
     return std::isinf(x);
 }
+
+float util_float_infinity() {
+    return std::numeric_limits<double>::infinity();
+}
+
+double util_double_infinity() {
+    return std::numeric_limits<double>::infinity();
+}
