@@ -5,16 +5,9 @@
 #include <QStandardPaths>
 #include <QtGlobal>
 
-// shout.h checks for WIN32 to see if we are on Windows.
-#ifdef WIN64
-#define WIN32
-#endif
 #ifdef __BROADCAST__
 #include <shoutidjc/shout.h>
 #endif
-#ifdef WIN64
-#undef WIN32
-#endif
 
 #ifdef __RUBBERBAND__
 #include <rubberband/RubberBandStretcher.h>
@@ -22,6 +15,7 @@
 
 #include <FLAC/format.h>
 #include <chromaprint.h>
+#include <ebur128.h>
 #include <lame/lame.h>
 #include <portaudio.h>
 #include <sndfile.h>
@@ -85,6 +79,8 @@ QString VersionStore::platform() {
     QString base = QStringLiteral("FreeBSD");
 #elif defined(__BSD__)
     QString base = QStringLiteral("BSD");
+#elif defined(__EMSCRIPTEN__)
+    QString base = QStringLiteral("Emscripten");
 #else
     QString base = QStringLiteral("Unknown OS");
 #endif
@@ -111,6 +107,10 @@ QString VersionStore::platform() {
         defined(__PPC64__) || defined(_ARCH_PPC) || defined(_ARCH_PPC64) ||   \
         defined(_M_PPC)
     base.append(" PowerPC");
+#elif defined(__wasm32__)
+    base.append(" Wasm32");
+#elif defined(__wasm__)
+    base.append(" Wasm");
 #endif
 
     return base;
@@ -141,6 +141,11 @@ QString VersionStore::gitVersion() {
     return gitVersion;
 }
 
+// static
+QString VersionStore::qtVersion() {
+    return qVersion();
+}
+
 // static
 QString VersionStore::buildFlags() {
     return kBuildFlags;
@@ -151,6 +156,12 @@ QStringList VersionStore::dependencyVersions() {
     sf_command(nullptr, SFC_GET_LIB_VERSION, sndfile_version, sizeof(sndfile_version));
     // Null-terminate just in case.
     sndfile_version[sizeof(sndfile_version) - 1] = '\0';
+
+    int eburMaj = 0;
+    int eburMin = 0;
+    int eburP = 0;
+    ebur128_get_version(&eburMaj, &eburMin, &eburP);
+
     // WARNING: may be inaccurate since some come from compile-time header
     // definitions instead of the actual dynamically loaded library).
     QStringList result;
@@ -181,6 +192,10 @@ QStringList VersionStore::dependencyVersions() {
                        .arg(QString::number(CHROMAPRINT_VERSION_MAJOR),
                                QString::number(CHROMAPRINT_VERSION_MINOR),
                                QString::number(CHROMAPRINT_VERSION_PATCH))
+            << QString("libebur128: %1.%2.%3")
+                       .arg(QString::number(eburMaj),
+                               QString::number(eburMin),
+                               QString::number(eburP))
             // Should be accurate.
             << QString("Vorbis: %1").arg(vorbis_version_string())
             // Should be accurate.
