@@ -3,13 +3,11 @@
 #include "util/workerthread.h"
 #include "util/fifo.h"
 
-
-class WorkerThread;
-
 /// Non-blocking scheduler for worker threads which itself runs
 /// as a worker thread. The maximum number of worker threads is
 /// limited.
 class WorkerThreadScheduler : public WorkerThread {
+    Q_OBJECT
   public:
     explicit WorkerThreadScheduler(
             int maxWorkers,
