@@ -1,21 +1,17 @@
 #pragma once
 
 #include <QColor>
-#include <QTime>
-//#include <QLinearGradient>
 
 #include "skin/legacy/skincontext.h"
 #include "util/class.h"
 #include "waveform/renderers/waveformrendererabstract.h"
-#include "waveform/waveformwidgetfactory.h"
 #include "util/performancetimer.h"
 
-class ControlObject;
 class ControlProxy;
 
 class WaveformRendererEndOfTrack : public WaveformRendererAbstract {
   public:
-    static const int s_maxAlpha = 125;
+    static constexpr int s_maxAlpha = 125;
     explicit WaveformRendererEndOfTrack(
             WaveformWidgetRenderer* waveformWidgetRenderer);
     virtual ~WaveformRendererEndOfTrack();
