@@ -5,6 +5,7 @@
 #include "waveform/renderers/allshader/waveformrendererendoftrack.h"
 #include "waveform/renderers/allshader/waveformrendererpreroll.h"
 #include "waveform/renderers/allshader/waveformrendererrgb.h"
+#include "waveform/renderers/allshader/waveformrendererslipmode.h"
 #include "waveform/renderers/allshader/waveformrendermark.h"
 #include "waveform/renderers/allshader/waveformrendermarkrange.h"
 #include "waveform/widgets/allshader/moc_rgbwaveformwidget.cpp"
@@ -20,6 +21,12 @@ RGBWaveformWidget::RGBWaveformWidget(const QString& group, QWidget* parent)
     addRenderer<WaveformRendererRGB>();
     addRenderer<WaveformRenderBeat>();
     addRenderer<WaveformRenderMark>();
+    // The following renderer will add an overlay waveform if a slip is in progress
+    addRenderer<WaveformRendererSlipMode>();
+    addRenderer<WaveformRendererPreroll>(::WaveformRendererAbstract::Slip);
+    addRenderer<WaveformRendererRGB>(::WaveformRendererAbstract::Slip);
+    addRenderer<WaveformRenderBeat>(::WaveformRendererAbstract::Slip);
+    addRenderer<WaveformRenderMark>(::WaveformRendererAbstract::Slip);
 
     m_initSuccess = init();
 }
