@@ -1,9 +1,8 @@
 #pragma once
 
+#include <QEvent>
 #include <QString>
 
-class QEvent;
-
 /// Mixin to mark a widget as a drop target for tracks.
 ///
 /// This class is *not* derived from QObject (inheriting from 2 QObject classes
@@ -22,8 +21,8 @@ class TrackDropTarget {
         emit trackDropped(filename, group); // clazy:exclude=incorrect-emit
     }
 
-    virtual bool handleDragAndDropEventFromWindow(QEvent* event) {
-        Q_UNUSED(event);
+    virtual bool handleDragAndDropEventFromWindow(QEvent* pEvent) {
+        pEvent->ignore();
         return false;
     }
 
