@@ -1,5 +1,9 @@
 #include "widget/wmainmenubar.h"
 
+#ifndef __APPLE__
+#include <QApplication>
+#include <QWindow>
+#endif
 #include <QUrl>
 
 #include "config.h"
@@ -7,6 +11,7 @@
 #include "defs_urls.h"
 #include "moc_wmainmenubar.cpp"
 #include "util/cmdlineargs.h"
+#include "util/desktophelper.h"
 #include "util/experiment.h"
 #include "vinylcontrol/defs_vinylcontrol.h"
 
@@ -81,6 +86,9 @@ WMainMenuBar::WMainMenuBar(QWidget* pParent, UserSettingsPointer pConfig,
 void WMainMenuBar::initialize() {
     // FILE MENU
     QMenu* pFileMenu = new QMenu(tr("&File"), this);
+#ifndef __APPLE__
+    connectMenuToSlotShowMenuBar(pFileMenu);
+#endif
 
     QString loadTrackText = tr("Load Track to Deck &%1");
     QString loadTrackStatusText = tr("Loads a track in deck %1");
@@ -129,6 +137,9 @@ void WMainMenuBar::initialize() {
 
     // LIBRARY MENU
     QMenu* pLibraryMenu = new QMenu(tr("&Library"), this);
+#ifndef __APPLE__
+    connectMenuToSlotShowMenuBar(pLibraryMenu);
+#endif
 
     QString rescanTitle = tr("&Rescan Library");
     QString rescanText = tr("Rescans library folders for changes to tracks.");
@@ -194,6 +205,33 @@ void WMainMenuBar::initialize() {
     QMenu* pViewMenu = new QMenu(tr("&View") + QStringLiteral("\u200C"), this);
 #else
     QMenu* pViewMenu = new QMenu(tr("&View"), this);
+    connectMenuToSlotShowMenuBar(pViewMenu);
+#endif
+
+#ifndef __APPLE__
+    // Show menu bar
+    QString showMenuBarTitle = tr("Auto-hide menu bar");
+    QString showMenuBarText = tr("Auto-hide the main menu bar when it's not used.");
+    auto* pViewAutoHideMenuBar = new QAction(showMenuBarTitle, this);
+    pViewAutoHideMenuBar->setCheckable(true);
+    pViewAutoHideMenuBar->setStatusTip(showMenuBarText);
+    pViewAutoHideMenuBar->setWhatsThis(buildWhatsThis(showMenuBarTitle, showMenuBarText));
+    connect(pViewAutoHideMenuBar,
+            &QAction::triggered,
+            this,
+            &WMainMenuBar::slotAutoHideMenuBarToggled);
+    // update checked state from config each time the menu is shown
+    connect(pViewMenu,
+            &QMenu::aboutToShow,
+            this,
+            [this, pViewAutoHideMenuBar]() {
+                bool autoHide = m_pConfig->getValue<bool>(
+                        ConfigKey("[Config]", "hide_menubar"), false);
+                pViewAutoHideMenuBar->setChecked(autoHide);
+            });
+    pViewMenu->addAction(pViewAutoHideMenuBar);
+
+    pViewMenu->addSeparator();
 #endif
 
     // Skin Settings Menu
@@ -277,7 +315,6 @@ void WMainMenuBar::initialize() {
             ConfigKey(kSkinGroup, QStringLiteral("show_library_coverart")));
     pViewMenu->addAction(pViewShowCoverArt);
 
-
     QString maximizeLibraryTitle = tr("Maximize Library");
     QString maximizeLibraryText = tr("Maximize the track library to take up all the available screen space.") +
             " " + mayNotBeSupported;
@@ -333,6 +370,9 @@ void WMainMenuBar::initialize() {
 
     // OPTIONS MENU
     QMenu* pOptionsMenu = new QMenu(tr("&Options"), this);
+#ifndef __APPLE__
+    connectMenuToSlotShowMenuBar(pOptionsMenu);
+#endif
 
 #ifdef __VINYLCONTROL__
     QMenu* pVinylControlMenu = new QMenu(tr("&Vinyl Control"), this);
@@ -452,6 +492,9 @@ void WMainMenuBar::initialize() {
     // DEVELOPER MENU
     if (CmdlineArgs::Instance().getDeveloper()) {
         QMenu* pDeveloperMenu = new QMenu(tr("&Developer"), this);
+#ifndef __APPLE__
+        connectMenuToSlotShowMenuBar(pDeveloperMenu);
+#endif
 
         QString reloadSkinTitle = tr("&Reload Skin");
         QString reloadSkinText = tr("Reload the skin");
@@ -553,6 +596,9 @@ void WMainMenuBar::initialize() {
 
     // HELP MENU
     QMenu* pHelpMenu = new QMenu(tr("&Help"), this);
+#ifndef __APPLE__
+    connectMenuToSlotShowMenuBar(pHelpMenu);
+#endif
 
     QString externalLinkSuffix;
 #ifndef __APPLE__
@@ -661,6 +707,32 @@ void WMainMenuBar::initialize() {
 
     pHelpMenu->addAction(pHelpAboutApp);
     addMenu(pHelpMenu);
+
+#ifndef __APPLE__
+    // Watch focus changes to hide the menubar as soon as all menus are closed,
+    // e.g. when an action was triggered or when all menus are closed by pressing
+    // Escape or clicking anywhere else
+    connect(qApp,
+            &QApplication::focusWindowChanged,
+            this,
+            [this]() {
+                if (!isNativeMenuBar() && height() > 0 && !activeAction()) {
+                    hideMenuBar();
+                }
+            });
+    // ... and when the focus widget changes (main window or dialogs)
+    connect(qApp,
+            // This would work, too, but unfortunately this is also emitted on
+            // leaveEvent of WStarRating in the library.
+            // &QApplication::focusObjectChanged,
+            &QApplication::focusChanged,
+            this,
+            [this]() {
+                if (!isNativeMenuBar() && height() > 0 && !activeAction()) {
+                    hideMenuBar();
+                }
+            });
+#endif
 }
 
 void WMainMenuBar::onKeywheelChange(int state) {
@@ -701,9 +773,91 @@ void WMainMenuBar::onDeveloperToolsHidden() {
 }
 
 void WMainMenuBar::onFullScreenStateChange(bool fullscreen) {
+#ifndef __APPLE__
+    // always try to hide the menubar when we switched
+    hideMenuBar();
+#endif
     emit internalFullScreenStateChange(fullscreen);
 }
 
+#ifndef __APPLE__
+void WMainMenuBar::connectMenuToSlotShowMenuBar(const QMenu* pMenu) {
+    // If a menu hotkey like Alt+F(ile) is pressed while the menubar is hidden,
+    // show the menubar and open the requested menu. See showMenuBar() for details.
+
+    // NOTE(ronso0) Test with xfwm4 and other window managers if you change this
+    // code or think you found alternative ways to toggle the menubar!
+    // In Gnome for example, when highlighting (pressed Alt only) or activating
+    // menus (Alt combos), the menubar receives focusIn events (even while it is
+    // hidden), and focusOut events respectively when closing menus. Hence we
+    // could simply show/hide the menubar in QMenuBar::focusInEvent() and focusoutEvent().
+    // However, with xfwm4 for example, menus are activated directly, i.e. the
+    // menubar doesn't receive focus change events.
+    connect(pMenu,
+            &QMenu::aboutToShow,
+            this,
+            [this]() {
+                if (!isNativeMenuBar() && height() <= 0) {
+                    showMenuBar();
+                }
+            });
+}
+
+void WMainMenuBar::slotToggleMenuBar() {
+    if (isNativeMenuBar()) {
+        return;
+    }
+
+    if (height() > 0) {
+        hideMenuBar();
+    } else {
+        showMenuBar();
+    }
+}
+
+void WMainMenuBar::showMenuBar() {
+    if (isNativeMenuBar()) {
+        return;
+    }
+    // Note: the resulting resizeEvent() calls QMenuBarPrivate::updateGeometries
+    // which resets currentAction() to nullptr. So in case the menubar is shown
+    // in response to a specific menu hotkey (Alt-F), or pressing Alt opens the
+    // first menu (File) (depends on OS / window manager), the current action
+    // will be reset to null and menus can't be switched with Left/Right keys
+    // anymore, i.e. we'd be stuck in the triggered menu.
+    // Workaround: reselect the active action after unhiding. It can be none,
+    // user-requested (hotkey) or auto-selected (first menu's first action).
+    QAction* pAct = activeAction();
+    setMinimumHeight(sizeHint().height());
+    // If there was a menu selected before, reselect that.
+    // Note: with nullptr this would be a no-op. Though, even if no menu is
+    // selected, hovering a menu would instantly open that (no click required).
+    if (pAct) {
+        setActiveAction(pAct);
+    }
+    // TODO Alternatively, activate the first menu?
+    // setActiveAction(pAct ? pAct : actions().first());
+}
+
+void WMainMenuBar::hideMenuBar() {
+    if (isNativeMenuBar()) {
+        return;
+    }
+    if (m_pConfig->getValue<bool>(ConfigKey("[Config]", "hide_menubar"), false)) {
+        // don't use setHidden(true) because Alt hotkeys wouldn't work anymore
+        setFixedHeight(0);
+    }
+}
+
+void WMainMenuBar::slotAutoHideMenuBarToggled(bool autoHide) {
+    m_pConfig->setValue(ConfigKey("[Config]", "hide_menubar"), autoHide ? 1 : 0);
+    // Just in case it was hidden after toggling the menu action
+    if (!autoHide) {
+        showMenuBar();
+    }
+}
+#endif
+
 void WMainMenuBar::onVinylControlDeckEnabledStateChange(int deck, bool enabled) {
     VERIFY_OR_DEBUG_ASSERT(deck >= 0 && deck < m_vinylControlEnabledActions.size()) {
         return;
@@ -733,7 +887,7 @@ void WMainMenuBar::slotDeveloperDebugger(bool toggle) {
 }
 
 void WMainMenuBar::slotVisitUrl(const QUrl& url) {
-    QDesktopServices::openUrl(url);
+    mixxx::DesktopHelper::openUrl(url);
 }
 
 void WMainMenuBar::createVisibilityControl(QAction* pAction,
