@@ -18,6 +18,7 @@
 
 class DlgTagFetcher;
 class DlgTrackInfo;
+class DlgTrackInfoMulti;
 //class DlgDeleteFilesConfirmation;
 class ExternalTrackCollection;
 class Library;
@@ -26,6 +27,7 @@ class WColorPickerAction;
 class WCoverArtMenu;
 class WFindOnWebMenu;
 class WSearchRelatedTracksMenu;
+class WStarRatingAction;
 
 /// A context menu for track(s).
 /// Can be used with individual track type widgets based on TrackPointer
@@ -97,14 +99,19 @@ class WTrackMenu : public QMenu {
 
     void loadTrack(
             const TrackPointer& pTrack, const QString& deckGroup);
+    /// Set the track property string which can be used to tell DlgTrackInfo/~Multi
+    /// to focus a specific metadata editor widget
+    void setTrackPropertyName(const QString& property = QString()) {
+        m_trackProperty = property;
+    }
 
     // WARNING: This function hides non-virtual QMenu::popup().
     // This has been done on purpose to ensure menu doesn't popup without loaded track(s).
     void popup(const QPoint& pos, QAction* at = nullptr);
     void slotShowDlgTrackInfo();
-    void showDlgTrackInfo(const QString& property = QString());
     // Library management
     void slotRemoveFromDisk();
+    const QString getDeckGroup() const;
 
   signals:
     void loadTrackToPlayer(TrackPointer pTrack, const QString& group, bool play = false);
@@ -117,6 +124,9 @@ class WTrackMenu : public QMenu {
     void slotOpenInFileBrowser();
     void slotSelectInLibrary();
 
+    // Track rating
+    void slotSetRating(int rating);
+
     // Row color
     void slotColorPicked(const mixxx::RgbColor::optional_t& color);
 
@@ -145,6 +155,7 @@ class WTrackMenu : public QMenu {
     void slotLockBpm();
     void slotUnlockBpm();
     void slotScaleBpm(mixxx::Beats::BpmScale scale);
+    void slotUndoBeatsChange();
 
     // Info and metadata
     void slotUpdateReplayGainFromPregain();
@@ -183,6 +194,7 @@ class WTrackMenu : public QMenu {
     QList<TrackRef> getTrackRefs() const;
 
     TrackPointer getFirstTrackPointer() const;
+    TrackPointerList getTrackPointers() const;
 
     std::unique_ptr<mixxx::TrackPointerIterator> newTrackPointerIterator() const;
 
@@ -221,6 +233,11 @@ class WTrackMenu : public QMenu {
     void clearTrackSelection();
 
     std::pair<bool, bool> getTrackBpmLockStates() const;
+    bool canUndoBeatsChange() const;
+
+    /// Get the common rating of all selected tracks.
+    /// Return 0 if ratings differ.
+    int getCommonTrackRating() const;
 
     /// Get the common track color of all tracks this menu is shown for, or
     /// return `nullopt` if there is no common color. Tracks may have no color
@@ -302,8 +319,10 @@ class WTrackMenu : public QMenu {
     QAction* m_pBpmFourThirdsAction{};
     QAction* m_pBpmThreeHalvesAction{};
     QAction* m_pBpmResetAction{};
+    QAction* m_pBpmUndoAction{};
 
-    // Track color
+    // Track rating and color
+    WStarRatingAction* m_pStarRatingAction{};
     WColorPickerAction* m_pColorPickerAction{};
 
     // Analysis actions
@@ -335,6 +354,7 @@ class WTrackMenu : public QMenu {
     PollingControlProxy m_pNumPreviewDecks;
 
     std::unique_ptr<DlgTrackInfo> m_pDlgTrackInfo;
+    std::unique_ptr<DlgTrackInfoMulti> m_pDlgTrackInfoMulti;
     std::unique_ptr<DlgTagFetcher> m_pDlgTagFetcher;
 
     struct UpdateExternalTrackCollection {
@@ -349,6 +369,8 @@ class WTrackMenu : public QMenu {
 
     Features m_eActiveFeatures;
     const Features m_eTrackModelFeatures;
+
+    QString m_trackProperty;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(WTrackMenu::Features)
